import { messageHistory, recordMessage } from "./messages.js";
import { sendFeedback } from "./feedback.js";

const BACKEND_URL = 'https://chat.upb.ro/api/chat/';

function enableInputControls() {
    const chatInput = document.getElementById('chat-input');
    chatInput.disabled = false;
    chatInput.placeholder = "Type your message...";

    const chatSubmit = document.getElementById('chat-submit');
    chatSubmit.disabled = false;
    chatSubmit.classList.remove('opacity-50', 'cursor-not-allowed');
}

function disableInputControls() {
    const chatInput = document.getElementById('chat-input');
    chatInput.disabled = true;
    chatInput.placeholder = "Please wait for a response...";

    const chatSubmit = document.getElementById('chat-submit');
    chatSubmit.disabled = true;
    chatSubmit.classList.add('opacity-50', 'cursor-not-allowed');
}

function removeTypingIndicator() {
    const typingIndicator = document.getElementById('typing-indicator');
    if (typingIndicator) {
        typingIndicator.remove();
    }
}

async function handleStreamingResponse(stream) {
    const reader = stream.getReader();
    const decoder = new TextDecoder();
    let assistantMessage = "";

    const replyElement = document.createElement('div');
    replyElement.className = 'flex mb-3';
    const messageContent = document.createElement('div');
    messageContent.className = 'message-content bg-gray-200 text-black rounded-lg py-2 px-4 max-w-[70%]';
    replyElement.appendChild(messageContent);

    const chatMessages = document.getElementById('chat-messages');
    chatMessages.appendChild(replyElement);
    chatMessages.scrollTop = chatMessages.scrollHeight;

    try {
        while (true) {
            const { value, done } = await reader.read();
            if (done) break;

            const textChunk = decoder.decode(value, { stream: true });

            assistantMessage += textChunk;

            let html = marked.parse(assistantMessage);

            html = DOMPurify.sanitize(html, { ADD_ATTR: ['target'] });

            const tempElement = document.createElement('div');
            tempElement.innerHTML = html;
            tempElement.querySelectorAll('a').forEach(link => {
                link.setAttribute('target', '_blank');
                link.setAttribute('rel', 'noopener noreferrer');
            });

            messageContent.innerHTML = tempElement.innerHTML;

            removeTypingIndicator();
            chatMessages.scrollTop = chatMessages.scrollHeight;
        }

        recordMessage({ role: "assistant", content: assistantMessage });

        // Append feedback bar
        removePreviousFeedback();

        const feedbackBar = document.createElement('div');
        feedbackBar.className = 'feedback-bar';
        feedbackBar.innerHTML = `
          <span>Vi se pare util acest răspuns?</span>
          <button class="feedback-button" data-action="like">👍</button>
          <button class="feedback-button" data-action="dislike">👎</button>
        `;

        const index = messageHistory.length - 1;
        feedbackBar.querySelectorAll('.feedback-button').forEach(btn => {
          btn.addEventListener('click', () => {
            const action = btn.dataset.action;
            sendFeedback(index, action);
            feedbackBar.innerHTML = `<span>Mulțumim pentru feedback!</span>`;
          });
        });

        messageContent.appendChild(feedbackBar);

    } catch (error) {
        console.error('Error processing stream:', error);
    } finally {
        removeTypingIndicator();
        enableInputControls();
    }
}

async function fetchOpenAIResponse(message) {
    try {
        // Prepare the request payload
        const requestOptions = {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json'
            },
            body: JSON.stringify({ messages: messageHistory,
                temperature: 0.7
            })
        };

        const response = await fetch(BACKEND_URL, requestOptions);

        if (response.status === 422) {
            removeTypingIndicator();
            reply("Te rog refrazează întrebarea. Nu am înțeles ce dorești să afli.");
        } else if (!response.ok) {
            throw new Error(`API request failed with status ${response.status}`);
        } else {
            return await handleStreamingResponse(response.body);
        }

    } catch (error) {
        console.error('Error fetching response:', error);
        removeTypingIndicator();
        reply("Îmi pare rău, nu am putut procesa solicitarea dumneavoastră. Vă rugăm să încercați mai târziu.");
    }
}

function removePreviousFeedback() {
    document.querySelectorAll('.feedback-bar').forEach(el => el.remove());
}

export function reply(message) {
    removePreviousFeedback(); // remove existing feedback UI

    const index = messageHistory.length; // this will be the index for the assistant message

    const replyElement = document.createElement('div');
    replyElement.className = 'flex mb-3 relative';

    const messageContent = document.createElement('div');
    messageContent.className = 'message-content bg-gray-200 text-black rounded-lg py-2 px-4 max-w-[70%]';
    messageContent.innerHTML = message;

    const feedbackBar = document.createElement('div');
    feedbackBar.className = 'feedback-bar';
    feedbackBar.innerHTML = `
      <span>Vi se pare util acest răspuns?</span>
      <button class="feedback-button" data-action="like">👍</button>
      <button class="feedback-button" data-action="dislike">👎</button>
    `;

    feedbackBar.querySelectorAll('.feedback-button').forEach(btn => {
      btn.addEventListener('click', () => {
        const action = btn.dataset.action;
        sendFeedback(index, action);
        feedbackBar.innerHTML = `<span>Mulțumim pentru feedback!</span>`;
      });
    });

    messageContent.appendChild(feedbackBar);
    replyElement.appendChild(messageContent);

    const chatMessages = document.getElementById('chat-messages');
    chatMessages.appendChild(replyElement);
    chatMessages.scrollTop = chatMessages.scrollHeight;
}

function showTypingIndicator() {
    const typingElement = document.createElement('div');
    typingElement.id = 'typing-indicator';
    typingElement.className = 'flex mb-3';
    typingElement.innerHTML = `
        <div class="bg-gray-200 text-black rounded-lg py-2 px-4">
          <div class="typing-indicator">
            <span></span>
            <span></span>
            <span></span>
          </div>
        </div>
      `;

    const chatMessages = document.getElementById('chat-messages');
    chatMessages.appendChild(typingElement);
    chatMessages.scrollTop = chatMessages.scrollHeight;
}

export async function onUserRequest(message) {
    const messageElement = document.createElement('div');
    messageElement.className = 'flex justify-end mb-3';
    messageElement.innerHTML = `
      <div class="bg-gray-800 text-white rounded-lg py-2 px-4 max-w-[70%]">
        ${message}
      </div>
    `;

    const chatMessages = document.getElementById('chat-messages');
    chatMessages.appendChild(messageElement);
    chatMessages.scrollTop = chatMessages.scrollHeight;

    messageHistory.push({ role: "user", content: message });

    showTypingIndicator();
    disableInputControls();

    try {
        await fetchOpenAIResponse(message);

    } catch (error) {
        console.error('Error fetching response:', error);
        removeTypingIndicator();
        reply("Îmi pare rău, nu am putut procesa solicitarea dumneavoastră. Vă rugăm să încercați mai târziu.");
    } finally {
        enableInputControls();
    }
}
